# -*- coding: utf-8 -*-

import concurrent.futures
from logging import getLogger
import time

from lib import status
from lib import protocol

class TCPDownload:
    """同時並行でいくつものTCPダウンロードを進めるクラス"""
    
    def __init__(self, args, *, logger=None):
        """コンストラクタ。argsから実行パラメータを取得する"""
        # 同時にダウンロード処理を実行する個数
        self._concurrent = args.concurrent
        if logger is None:
            self._logger = getLogger(__main__)
        else:
            self._logger = logger
    
    def execute(self, imageFile, targetHosts, errorFile):
        """与えられたホスト群に対してTCPダウンロード処理を実行する"""
        with concurrent.futures.ThreadPoolExecutor(max_workers = self._concurrent) \
                as executor:
            futures = {executor.submit(self._download, imageFile, targetHost): targetHost for targetHost in targetHosts} 
            
            # 全部の実行結果を調べる
            errorList = []
            for future in concurrent.futures.as_completed(futures):
                host = futures[future]
                hostport = host.getHostPortStr()
                try:
                    data = future.result()
                except Exception as exc:
                    self._logger.error("{0} {1}".format(hostport, exc))
                    errorList.append(host)
                else:
                    self._logger.info("{0} succeed.".format(hostport))
        
        return errorList

    def _download(self, imageFile, targetHost):
        """単一のホストに対してTCPダウンロード処理を実行する"""
        tdp = protocol.TCPDownloadProtocol(imageFile, targetHost)
        tdp.download()
