# -*- coding: utf-8 -*-

import ipaddress
from lib import status

class TargetHost:
    """TCPダウンロードの対象となるホスト"""
    
    def __init__(self, strline):
        """コンストラクタ。strlineはIPアドレスもしくはIPアドレス:ポート番号の形式のstr。"""
        (ipAddress, port) = self._getAddrFromLine(strline)

        # IPアドレスが指定の形式で無かったらException
        self._ipAddress = ipaddress.ip_address(ipAddress)

        if (1 <= port) and (port <= 65535):
            self._port = port
        else:
            raise ValueError()
        
        self.hostStatus = status.HostStatus.NOTSTART
        self.chunkSent = -1;
        self.chunkDone = -1;
    
    def getHostPort(self):
        """IPアドレスとポートの組(タプル)を取得する"""
        return (str(self._ipAddress), self._port)
    
    def getHostPortStr(self):
        """IPアドレス:ポートを文字列で返す"""
        return "{0}:{1}".format(self._ipAddress, self._port)

    def confirmSentChunk(self):
        """送ったchunkが正常に送れたことが確認できた場合に呼ぶ。TargetHost内部の送ったchunk番号を更新する。"""
        self.chunkDone = self.chunkSent

    def getNextChunk(self):
        """次に送るchunkのインデックスを取り出す"""
        return self.chunkSent + 1
    
    def __str__(self):
        return "TargetHost: ipaddr={0}, port={1}, hostStatus={2}, chunkDone={3}"\
            .format(self._ipAddress, self._port, self.hostStatus, self.chunkDone)
    
    def _getAddrFromLine(self, line):
        """1行の文字列(IP:PORT)から、IPアドレスとポート番号を取得する"""
        if not isinstance(line, str):
            raise TypeError()
        
        l = line.strip().split(":")
        
        if len(l) == 1:
            ipAddress = l[0]
            port = 2222
        elif len(l) == 2:
            ipAddress = l[0]
            port = int(l[1])
        else:
            raise ValueError()
        
        return (ipAddress, port)
    
    @staticmethod
    def getTargetHosts(filename):
        """IP:PORTの組が書いてある複数行のテキストファイルから、TargetHostのリストとエラーになった行を返す"""
        targetHosts = []
        errorLines = []
        for line in open(filename):
            line = line.strip()
            if len(line) == 0 or line.startswith('#'):
                # 空行やコメント行なので飛ばす
                continue
            
            try:
                targetHost = TargetHost(line)
                targetHosts.append(targetHost)
            except Exception as ex:
                errorLines.append(line)
        
        return (targetHosts, errorLines)

def testTargetHosts():
    (targetHosts, errorLines) = TargetHost.getTargetHosts("list.txt")
    for targetHost in targetHosts:
        print(targetHost)
    
    print("---")
    
    for errorLine in errorLines:
        print(errorLine)

if __name__ == "__main__":
    testTargetHosts()